/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.api.controller;

import com.alipay.api.AlipayClient;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.common.constant.CommonConstant;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.core.tool.utils.WebUtil;
import org.springblade.modules.api.request.OrderCommonSaveRequest;
import org.springblade.modules.api.request.OrderConsignRandomRequest;
import org.springblade.modules.api.request.OrderTakeRequest;
import org.springblade.modules.api.response.ConsignResponse;
import org.springblade.modules.auth.utils.TokenUtil;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.service.IOrderService;
import org.springblade.modules.finance.vo.OrderVO;
import org.springblade.modules.finance.wrapper.OrderWrapper;
import org.springblade.modules.shopping.entity.Consign;
import org.springblade.modules.shopping.service.IConsignService;
import org.springblade.modules.shopping.vo.ConsignVO;
import org.springblade.modules.shopping.wrapper.ConsignWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 控制器
 *
 * @author xuwei
 * @since 2021-12-23
 */
@RestController
@AllArgsConstructor
@RequestMapping(CommonConstant.API_URL)
@Api(value = "订单模块", tags = "05.订单模块")
public class Api05Controller extends BladeController {

	private final IOrderService orderService;
	private final IConsignService consignService;
	@Autowired
	private AlipayClient client;//支付宝请求sdk客户端

	@PostMapping("/finance/order-commonSave")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "购买商品", notes = "传入用户ID")
	public R orderCommonSave(@Valid @RequestBody OrderCommonSaveRequest request) {
		String tenantId = WebUtil.getRequest().getHeader(TokenUtil.TENANT_HEADER_KEY);
		if(StringUtil.isEmpty(request.getDistributeMode())){
			request.setDistributeMode(2); //物流配送
		}
		if(StringUtil.isEmpty(request.getUserId())){
			return R.fail("请求参数错误！");
		}
		if(request.getGoodsList() == null || request.getGoodsList().size() == 0){
			return R.fail("请选择购买的商品！");
		}
		return R.data(orderService.commonOrderInsert(request, tenantId));
	}

	@PostMapping("/finance/order-consignRandom")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "寄售抢购", notes = "抢购订单")
	public R consignRandom(@RequestBody OrderConsignRandomRequest request) {
		String tenantId = WebUtil.getRequest().getHeader(TokenUtil.TENANT_HEADER_KEY);
		ConsignResponse response = orderService.consignOrderInsert(request, tenantId);
		if (StringUtil.isEmpty(response.getOutTradeNo())){
			if (response.getStatus() == 1){
				return R.data(400,response,"");
			}
		}
		return R.data(response);
	}

	@GetMapping("/finance/order-page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "我的订单", notes = "传入用户ID，支付状态 0:全部，1待付款，2:待发货，3待收货，4已完成，5已取消")
	public R<List<OrderVO>> orderPage(@ApiParam(value = "用户ID", required = true) @RequestParam Long userId,
									  @ApiParam(value = "支付状态", required = false) @RequestParam(required = false) Integer payFlag,
									  Query query) {
		Order order = new Order();
		order.setUserId(userId);
		if(!StringUtil.isEmpty(payFlag)){
			if(payFlag != 0){
				order.setPayFlag(payFlag); //订单状态
			}
		}
		IPage<Order> pages = orderService.page(Condition.getPage(query),
			Condition.getQueryWrapper(order).orderByDesc("create_time"));
		return R.data(OrderWrapper.build().listVO(pages.getRecords()));
	}

	@GetMapping("/finance/consign-selfPage")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "我的仓单", notes = "传入用户ID，状态: 1寄售中，2:待收款，3:已收款, 不传或者传其他则查询全部")
	public R<List<ConsignVO>> consignPage(@ApiParam(value = "用户ID", required = true) @RequestParam(required = true) Long userId,
										  @ApiParam(value = "状态", required = false) @RequestParam(required = false) Integer status,
										  Query query) {
		Consign consign = new Consign();
		consign.setConsignUserId(userId);
		if(status == 1){
			consign.setConsignState(1); //寄售中
		}else if(status == 2){
			consign.setConsignState(2); //待收款
		}else if(status == 3){
			consign.setConsignState(3); //已收款
		}//查询全部
		IPage<Consign> pages = consignService.page(Condition.getPage(query), Condition.getQueryWrapper(consign));
		return R.data(ConsignWrapper.build().listVO(pages.getRecords()));
	}

	@PostMapping("/finance/consign-selfTake")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "仓单提货", notes = "我的仓单提货")
	public R consignTake(@Valid @RequestBody OrderTakeRequest request) {
		return R.data(orderService.consignSelfTake(request));
	}

	@PostMapping("/finance/order-cancel")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "取消订单", notes = "传入订单号")
	public R orderCancel(@ApiParam(value = "订单ID", required = true) @RequestParam Long id) {
		Order order = new Order();
		order.setId(id);
		order.setPayFlag(5); //已取消
		return R.status(orderService.updateById(order));
	}

	@PostMapping("/finance/order-confirm")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "确认收货", notes = "待收货变为已完成")
	public R orderConfirm(@ApiParam(value = "订单ID", required = true) @RequestParam Long orderId,
					 @ApiParam(value = "用户id", required = true) @RequestParam(required = false) Long userId) {
		return R.status(orderService.orderEnd(orderId,userId));
	}

}
