/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.shopping.controller;

import io.swagger.annotations.*;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.constant.RoleConstant;
import org.springblade.core.tool.node.TreeNode;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.shopping.wrapper.CategoryWrapper;
import org.springblade.modules.system.vo.MenuVO;
import org.springblade.modules.system.wrapper.MenuWrapper;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.shopping.entity.Category;
import org.springblade.modules.shopping.vo.CategoryVO;
import org.springblade.modules.shopping.service.ICategoryService;
import org.springblade.core.boot.ctrl.BladeController;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;
import java.util.Map;

/**
 * 分类表 控制器
 *
 * @author BladeX
 * @since 2022-02-13
 */
@RestController
@AllArgsConstructor
@RequestMapping("shopping/category")
@Api(value = "分类表", tags = "分类表接口")
public class CategoryController extends BladeController {

	private final ICategoryService categoryService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入category")
	public R<Category> detail(Category category) {
		Category detail = categoryService.getOne(Condition.getQueryWrapper(category));
		return R.data(detail);
	}

	/**
	 * 懒加载列表
	 */
	@GetMapping("/lazy-list")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "懒加载列表", notes = "传入Category")
	public R<List<CategoryVO>> lazyList(Long parentId, @ApiIgnore @RequestParam Map<String, Object> menu) {
		List<CategoryVO> list = categoryService.lazyList(parentId, menu);
		return R.data(CategoryWrapper.build().listNodeLazyVO(list));
	}

	/**
	 * 获取树形结构
	 */
	@GetMapping("/tree")
	@ApiOperationSupport(order = 11)
	@ApiOperation(value = "树形结构", notes = "树形结构")
	public R<List<TreeNode>> tree() {
		List<TreeNode> tree = categoryService.tree();
		return R.data(tree);
	}

	/**
	 * 分页 分类表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入category")
	public R<IPage<Category>> list(Category category, Query query) {
		IPage<Category> pages = categoryService.page(Condition.getPage(query), Condition.getQueryWrapper(category));
		return R.data(pages);
	}

	/**
	 * 自定义分页 分类表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入category")
	public R<IPage<CategoryVO>> page(CategoryVO category, Query query) {
		IPage<CategoryVO> pages = categoryService.selectCategoryPage(Condition.getPage(query), category);
		return R.data(pages);
	}

	/**
	 * 新增 分类表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入category")
	public R save(@Valid @RequestBody Category category) {
		return R.status(categoryService.save(category));
	}

	/**
	 * 修改 分类表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入category")
	public R update(@Valid @RequestBody Category category) {
		return R.status(categoryService.updateById(category));
	}

	/**
	 * 新增或修改 分类表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入category")
	public R submit(@Valid @RequestBody Category category) {
		return R.status(categoryService.saveOrUpdate(category));
	}


	/**
	 * 删除 分类表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(categoryService.removeByIds(Func.toLongList(ids)));
	}


}
